begin
for x in (select table_name from user_tables where table_name in ('WOHNORT','PERSON','ADRESSE') order by 1 desc) loop
    execute immediate 'drop table ' || x.table_name || ' cascade constraints';
end loop;
end;
/

create table Person (
  PersonID NUMBER(10) not null
, Name VARCHAR2(100 char)
, Vorname VARCHAR2(100 char)
, Geburtstag DATE
, Kinder NUMBER(2)
, constraint PersonPK primary key(PersonID) RELY
); 

COMMENT ON TABLE Person IS 'Diese Tabelle enthält Personendaten, die über eine eindeutige PersonID referenziert werden können.';

create table Adresse (
  AdressID NUMBER(10) not null
, Postleitzahl VARCHAR2(100 char)
, Stadt VARCHAR2(100 char)
, Strasse VARCHAR2(100 char)
, Hausnummer VARCHAR2(100 char)
, constraint AdressePK primary key(AdressID) RELY
, constraint AdresseUQ1 unique(Postleitzahl, Stadt, Strasse,Hausnummer) RELY
);

COMMENT ON TABLE Adresse IS 'Diese Tabelle enthält Adressen, die über die eindeutige AdressID referenziert werden können.';
COMMENT ON COLUMN Adresse.Postleitzahl IS 'Postleitzahl.';
COMMENT ON COLUMN Adresse.Stadt IS 'Stadt.';
COMMENT ON COLUMN Adresse.Strasse IS 'Straßenname.';
COMMENT ON COLUMN Adresse.Hausnummer IS 'Hausnummer.';

create table Wohnort (
  PersonID NUMBER(10) not null
, AdressID NUMBER(10) not null
, constraint WohnortPK primary key(PersonID, AdressID) RELY
, constraint WohnortFK1 foreign key(PersonID)references Person(PersonID) ENABLE
, constraint WohnortFK2 foreign key(AdressID)references Adresse(AdressID) ENABLE
);

COMMENT ON TABLE Wohnort IS 'Über diese Tabelle können Personen einer Adresse zugeordnet werden.';

create or replace view Bewohner as 
select AdressID,Bewohner
from (select AdressID,to_clob(listagg(Vorname|| ' ' || Name, '
') within group(order by Name, Vorname)) Bewohner from Wohnort join Person using(PersonID) group by AdressID)
join Adresse using(AdressID)
;

COMMENT ON TABLE Bewohner IS 'Diese View stellt für jede AdressID eine Liste der Vor- und Zunamen der dort wohnenden Personen bereit.';
